/** 
 * 
 * Copyright 2004 Hiram Chirino
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License. 
 * 
 **/
package org.activeio.adapter;

import java.io.IOException;
import java.net.URI;

import org.activeio.AsynchChannelFactory;
import org.activeio.SynchChannel;
import org.activeio.SynchChannelFactory;
import org.activeio.SynchChannelServer;

/**
 * @version $Revision$
 */
final public class AsynchToSynchChannelFactoryAdaptor implements SynchChannelFactory {
    
    private AsynchChannelFactory asynchChannelFactory;
    
    static public SynchChannelFactory adapt(AsynchChannelFactory channelFactory ) {

        // It might not need adapting
        if( channelFactory instanceof SynchChannelServer ) {
            return (SynchChannelFactory) channelFactory;
        }

        // Can we just just undo the adaptor
        if( channelFactory.getClass() == SynchToAsynchChannelFactoryAdaptor.class ) {
            return ((SynchToAsynchChannelFactoryAdaptor)channelFactory).getSynchChannelFactory();
        }
        
        return new AsynchToSynchChannelFactoryAdaptor((AsynchChannelFactory)channelFactory);        
    }
    
    
    private AsynchToSynchChannelFactoryAdaptor(AsynchChannelFactory asynchChannelFactory) {
        this.asynchChannelFactory = asynchChannelFactory;
    }
        
    public SynchChannel openSynchChannel(URI location) throws IOException {
        return AsynchToSynchChannelAdapter.adapt( asynchChannelFactory.openAsynchChannel(location) );
    }
    
    public SynchChannelServer bindSynchChannel(URI location) throws IOException {
        return AsynchToSynchChannelServerAdapter.adapt(asynchChannelFactory.bindAsynchChannel(location));
    }
    
    public AsynchChannelFactory getAsynchChannelFactory() {
        return asynchChannelFactory;
    }
}
