/**
 * 
 * Copyright 2004 Hiram Chirino
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License. 
 * 
 **/

package org.activeio.net;

import java.io.IOException;
import java.net.Socket;
import java.net.URI;
import java.nio.channels.ServerSocketChannel;

import org.activeio.AsynchChannel;
import org.activeio.Channel;
import org.activeio.filter.WriteBufferedAsynchChannel;
import org.activeio.packet.ByteBufferPacket;

/**
 * A SynchChannelServer that creates
 * {@see org.activeio.net.TcpSynchChannel}objects from accepted
 * tcp socket connections.
 * 
 * @version $Revision$
 */
public class NIOAsynchChannelServer extends SocketSynchChannelServer {

    private final boolean createWriteBufferedChannels;
	private final boolean useDirectBuffers;

    public NIOAsynchChannelServer(ServerSocketChannel socketChannel, URI bindURI, URI connectURI, boolean createWriteBufferedChannels, boolean useDirectBuffers) {
        super(socketChannel.socket(), bindURI, connectURI);
        this.createWriteBufferedChannels = createWriteBufferedChannels;
		this.useDirectBuffers = useDirectBuffers;
    }
    
    protected Channel createChannel(Socket socket) throws IOException {
        AsynchChannel channel = new NIOAsynchChannel(socket.getChannel(), useDirectBuffers);
        if( createWriteBufferedChannels ) {
            channel = new WriteBufferedAsynchChannel(channel, ByteBufferPacket.createDefaultBuffer(useDirectBuffers), false);
        }
        return channel;
    }
}